package com.nisum.Employeeinfo.dao;

import com.nisum.Employeeinfo.errorcode.ApiErrorCode;
import com.nisum.Employeeinfo.exception.EmployeeNotFoundException;
import com.nisum.Employeeinfo.model.Employee;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.r2dbc.core.DatabaseClient;
import org.springframework.stereotype.Repository;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Repository
public class EmployeeDao {

    @Autowired
    private DatabaseClient databaseClient;

    public Mono<Employee> saveEmployee(Employee employee){
        String query="INSERT INTO employee ( name, email, dept, sal) VALUES (:name, :email, :dept, :sal)";
        return databaseClient.sql(query)

                .bind("name",employee.getName())
                .bind("email",employee.getEmail())
                .bind("dept",employee.getDept())
                .bind("sal",employee.getSal())
                .fetch()
                .rowsUpdated()
                .flatMap(rows->{
                    if (rows>0){
                        return Mono.just(employee);
                    }else {
                        return Mono.error(new EmployeeNotFoundException(ApiErrorCode.EMPLOYEE_NOT_FOUND,"Employee Not Found"));
                    }
                });
    }
    public Flux<Employee> findAllEmployee(){
        String query="SELECT * FROM employee ";
        return databaseClient.sql(query)
                .map((row,metadata)-> new Employee(
                        row.get("id",Integer.class),
                        row.get("name", String.class),
                        row.get("email",String.class),
                        row.get("dept",String.class),
                        row.get("sal",Double.class)
                ))
                .all();

    }

    public Mono<Employee> findEmployeeById(Integer id){
        String query= "SELECT id,name,email,dept,sal FROM employee WHERE id = :id";

        return databaseClient.sql(query)
                .bind("id",id)
                .map((row, metadata) -> new Employee(
                        row.get("id", Integer.class),
                        row.get("name", String.class),
                        row.get("email", String.class),
                        row.get("dept", String.class),
                        row.get("sal", Double.class)
                ))
                .one();
    }
    public Mono<Employee> updateEmployee(Integer id,Employee employee){
        String query = "UPDATE employee SET name = :name, email = :email, dept = :dept, sal = :sal WHERE id = :id";

        return databaseClient
                .sql(query)
                .bind("id", id)
                .bind("name", employee.getName())
                .bind("email", employee.getEmail())
                .bind("dept", employee.getDept())
                .bind("sal", employee.getSal())
                .fetch()
                .rowsUpdated()
                .flatMap(updatedRows -> {
                    if (updatedRows > 0) {
                        // Return the updated employee object if update is successful
                        return Mono.just(employee);
                    } else {
                        // Return an error if no rows were updated (e.g., employee not found)
                        return Mono.error(new RuntimeException("Employee with ID " + id + " not found"));
                    }
                });
    }

    public Mono<Void> deleteEmployeeById(Integer id){
        String query= "DELETE FROM employee WHERE id = :id ";
        return databaseClient.sql(query)
                .bind("id",id)
                .fetch()
                .rowsUpdated()
                .flatMap(updateRow->{
                    if (updateRow>0){
                        return Mono.empty();
                    }else {
                      return  Mono.error(new EmployeeNotFoundException(ApiErrorCode.EMPLOYEE_NOT_FOUND,"Employee_Not_Found"));
                    }
                });
    }


}
